//
//
// step_dir_To_twoPhases_stepper.c
//
// Nicolas De Coster
// may 11 2018
//
// (c) ULB
// This work may be reproduced, modified, distributed,
// performed, and displayed for any purpose. Copyright is
// retained and must be preserved. The work is provided
// as is; no warranty is provided, and users accept all 
// liability.
//

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set


#define extpin_interrupt (1 << PCIE0)
#define extpin_interrupt_pin (1 << PCINT0)

#define PASTER(x,y) x ## y
#define CONCATENATE(x,y)  PASTER(x,y)


#define _M_PORT B
#define _IN1_PIN  0
#define _IN2_PIN  1
#define _IN3_PIN  2
#define _IN4_PIN  3
#define INTER_DELAY 10




#define _MOTOR_DDR    CONCATENATE(DDR, _M_PORT)
#define _MOTOR_PORT   CONCATENATE(PORT, _M_PORT)
#define _MOTOR_MASK  ((1<<_IN1_PIN)|(1<<_IN2_PIN)|(1<<_IN3_PIN)|(1<<_IN4_PIN))


#define _I_PORT   D
#define _INPUT_STEP   6
#define _INPUT_DIR    7

#define _INPUT_PINS   CONCATENATE(PIN, _I_PORT)
#define _INPUT_DDR    CONCATENATE(DDR, _I_PORT)
#define _INPUT_MASK   ((1<<_INPUT_STEP)|(1<<_INPUT_DIR))


#define CLOCKWISE         0
#define COUNTER_CLOCKWISE 1

#define ONE_TURN 20
#define STATES 4

unsigned char state[STATES]=
{
    (1<<_IN1_PIN) | (1<<_IN3_PIN),
    (1<<_IN1_PIN) | (1<<_IN4_PIN),
    (1<<_IN2_PIN) | (1<<_IN4_PIN),
    (1<<_IN2_PIN) | (1<<_IN3_PIN)
};

char steps_to_perform = 0;


// ISR(PCINT0_vect) {
//     if(bit_test(_INPUT_PINS, _INPUT_DIR))   steps_to_perform++;
//     else                                    steps_to_perform--;
// }


int main(void) {
   //
   // main
   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);

   
   
   _MOTOR_DDR |= _MOTOR_MASK;    //make motor pins output
   _MOTOR_PORT &= ~_MOTOR_MASK;  //put them all to zero
   
   _INPUT_DDR &= ~_INPUT_MASK;   //make input pins input
   
   
   //
   // set up pin change interrupt on input pin
   //
//    set(GIMSK, extpin_interrupt);
//    set (PCMSK0, extpin_interrupt_pin);
//    sei();
   //
   // main loop
   //
   while (1) {
    static unsigned char state_num = 0;
    unsigned char bLastStatus = 0;
    
    if(bit_test(_INPUT_PINS, _INPUT_STEP)){
        bLastStatus = 1;
        if(bit_test(_INPUT_PINS, _INPUT_DIR)){
            if(++state_num == STATES) state_num = 0;            
        }
        else{
            if(--state_num > STATES) state_num = STATES-1;
        }
    }
    
    _MOTOR_PORT = state[state_num] & _MOTOR_MASK;
    
    while(bLastStatus){
        if(!bit_test(_INPUT_PINS, _INPUT_STEP)) bLastStatus=0;
    }

//     if(steps_to_perform > 0){
//         if(++state_num == STATES) state_num = 0;
//         steps_to_perform--;
//     }
//     else if(steps_to_perform < 0){
//         if(--state_num > STATES) state_num = STATES-1;
//         steps_to_perform++;
//     }
//    _delay_ms(INTER_DELAY);
   }
}
